
package mobicomp.net;

import java.io.IOException;
import java.util.Collections;
import java.util.Observable;
import java.util.Observer;
import java.util.Vector;

import mobicomp.interfaces.MHMessage;
import mobicomp.interfaces.MHSocket;

/**
 * TODO
 * 
 * @author Andreas Wetzel <andi@spamato.net>
 */
public class MHMessageHandler implements Observer
{
    private MHSocketImpl socket;
    private MessageListener listener;
    
    public MHMessageHandler( MHSocketImpl socket, MessageListener listener) {
    		this.socket = socket;
    		this.listener = listener;
    }
    
    /**
     * @see ch.ethz.dcg.mobicomp.common.dispatcher.AbstractHandler#handleImpl(ch.ethz.dcg.mobicomp.common.dispatcher.Dispatchable)
     */
    public void update( Observable o, Object arg) {
    		MHMessage mhm = (MHMessage)arg;
    		if (mhm.getType() != MHMessage.TYPE_SRMSG) {
    			return;
    		}
        

    		if (mhm.getReceiver() == MHSocket.BROADCAST_ADDRESS) {
    			// notify upper layer of receipt
    			listener.onMessage( mhm);
    			
			// broadcast further...
    			int ttl = mhm.getTTL();
    			if ( (ttl > 1) || (ttl == 0) ) {
    				if ( ttl > 1 ) {
					mhm.setTTL( ttl - 1);
    				}
    				try {
					socket.sendMHMessage( mhm, MHSocket.BROADCAST_ADDRESS);
    				}
    				catch (IOException e) {
    					e.printStackTrace();
    				}
    			}
    		}
    		else if (mhm.getReceiver() == socket.getOwnAddress()) {
    			// this message is for me, send ack and notify listener
			MHMessage ack = new MHMessageImpl();
			ack.setSender( socket.getOwnAddress());
			ack.setReceiver( mhm.getSender());
			ack.setType( MHMessage.TYPE_SRACK);
			ack.setId( mhm.getId());

			Vector route = mhm.getRoute();
			Collections.reverse( route);
			ack.setRoute( route);
			ack.setRouteIndex( 1);
                    
			int nexthop = ((Integer)route.get( 1)).intValue();
			try {
//				System.out.println( socket.getOwnAddress() + ": sending ack to " + ack.getReceiver());
				socket.sendMHMessage( ack, nexthop);
			}
			catch (IOException e) {
				e.printStackTrace();
			}

			listener.onMessage( mhm);
		}
    		else {
    			// we just have to route eventually
    			Vector route = mhm.getRoute();
    			int index = mhm.getRouteIndex();
        	
    			// get intermediate receiver from the message route
    			int relay = ((Integer)route.get( index)).intValue();
    			// check if we are the 'relay' station
    			if (relay == socket.getOwnAddress()) {
    				// find next hop and submit message
    				int nexthop = ((Integer)route.get( index+1)).intValue();
    				mhm.setRouteIndex( index+1);
        			try {
					socket.sendMHMessage( mhm, nexthop);
				}
        			catch (IOException e) {
					e.printStackTrace();
				}
    			}
    		}        	
    }	
}
